#ifndef LIST_H_
#define LIST_H_

#include <stddef.h>

struct list_node
   {
   struct list_node * next;
   struct list_node * prev;
   void * data;
   };

struct list
   {
   struct list_node * head;
   struct list_node * tail;
   size_t size;
   };

struct list_iterator
   {
   struct list * l;
   struct list_node * curr;
   };

struct list * list_new(void);
void list_delete(struct list * l);

size_t list_size(struct list * l);

void * list_push_back(struct list * l, void * elem);
void * list_pop_back(struct list * l);
void * list_push_front(struct list * l, void * elem);
void * list_pop_front(struct list * l);


// The iterators are different that their c++ STL counterparts.
// They are bi-directional, and list_end points to the last element, not the empty space beyond the last elem.
void * list_begin(struct list * l, struct list_iterator * lit);
void * list_end(struct list * l, struct list_iterator * lit);
void * list_next(struct list_iterator * lit);
void * list_prev(struct list_iterator * lit);
void * list_erase(struct list_iterator * lit);
void * list_insert_after(struct list_iterator * lit, void * elem);
void * list_insert_before(struct list_iterator * lit, void * elem);
void * list_value(struct list_iterator * lit);

#endif
